(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('react'), require('prop-types'), require('backbone')) :
	typeof define === 'function' && define.amd ? define(['exports', 'react', 'prop-types', 'backbone'], factory) :
	(factory((global.ReactBackboneConnect = {}),global.React,global.PropTypes,global.Backbone));
}(this, (function (exports,React,PropTypes,Backbone) { 'use strict';

var React__default = 'default' in React ? React['default'] : React;
PropTypes = PropTypes && PropTypes.hasOwnProperty('default') ? PropTypes['default'] : PropTypes;
Backbone = Backbone && Backbone.hasOwnProperty('default') ? Backbone['default'] : Backbone;

var CONTEXT_KEY = '@BACKBONE_STORE';

// shallowEqual from fbjs.

var hasOwn = Object.prototype.hasOwnProperty;

function is(x, y) {
  if (x === y) {
    return x !== 0 || y !== 0 || 1 / x === 1 / y;
  } else {
    return x !== x && y !== y;
  }
}

function shallowEqual(objA, objB) {
  if (is(objA, objB)) return true;

  if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
    return false;
  }

  var keysA = Object.keys(objA);
  var keysB = Object.keys(objB);

  if (keysA.length !== keysB.length) return false;

  for (var i = 0; i < keysA.length; i++) {
    if (!hasOwn.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
      return false;
    }
  }

  return true;
}

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};









var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};











var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var _Provider$childContex;

var Provider = function (_PureComponent) {
  inherits(Provider, _PureComponent);

  function Provider() {
    classCallCheck(this, Provider);
    return possibleConstructorReturn(this, _PureComponent.apply(this, arguments));
  }

  Provider.prototype.getChildContext = function getChildContext() {
    var _ref;

    return _ref = {}, _ref[CONTEXT_KEY] = this.props.store, _ref;
  };

  Provider.prototype.render = function render() {
    return this.props.children;
  };

  return Provider;
}(React.PureComponent);

Provider.childContextTypes = (_Provider$childContex = {}, _Provider$childContex[CONTEXT_KEY] = PropTypes.object, _Provider$childContex);
Provider.propTypes = {
  children: PropTypes.node.isRequired,
  store: PropTypes.object
};

var getDisplayName = (function (component) {
  return component.displayName || component.name || 'Component';
});

var wrapModel = function wrapModel(curriedWrap, triggerChange, model) {
  var wrapped = Object.create(model);
  var keys = new Set();
  wrapped.get = function (propKey) {
    keys.add(propKey);
    return curriedWrap(triggerChange, model.get(propKey));
  };
  var changeListener = function changeListener(_ref) {
    var changed = _ref.changed;

    if (Object.keys(changed).some(function (key) {
      return keys.has(key);
    })) {
      triggerChange();
    }
  };

  model.on("change", changeListener);

  return {
    wrapped: wrapped,
    destory: function destory() {
      model.off("change", changeListener);
    }
  };
};

var wrapCollection = function wrapCollection(curriedWrap, triggerChange, collection) {
  var wrapped = Object.create(collection);
  wrapped.get = function (propKey) {
    return curriedWrap(triggerChange, collection.get(propKey));
  };
  wrapped.at = function (index) {
    return curriedWrap(triggerChange, collection.at(index));
  };
  wrapped.pop = function () {
    return curriedWrap(triggerChange, collection.pop());
  };
  wrapped.shift = function () {
    return curriedWrap(triggerChange, collection.shift());
  };
  wrapped.unshift = function () {
    return curriedWrap(triggerChange, collection.unshift());
  };

  collection.on("add remove reset", triggerChange);

  return {
    wrapped: wrapped,
    destory: function destory() {
      collection.off("add remove reset", triggerChange);
    }
  };
};

var wrap = function wrap(cache, triggerChange, object) {
  var wrapWithCache = function wrapWithCache(triggerChange, obj) {
    return wrap(cache, triggerChange, obj);
  };
  if (object instanceof Backbone.Model) {
    return cacheable(cache, object, function () {
      return wrapModel(wrapWithCache, triggerChange, object);
    }).wrapped;
  }
  if (object instanceof Backbone.Collection) {
    return cacheable(cache, object, function () {
      return wrapCollection(wrapWithCache, triggerChange, object);
    }).wrapped;
  }
  return object;
};

var cacheable = function cacheable(cache, key, fn) {
  var cached = cache.get(key);
  if (cached) {
    return cached;
  }
  var result = fn();
  cache.set(key, result);
  return result;
};

var StoreWatcher = function () {
  function StoreWatcher(store) {
    var _this = this;

    classCallCheck(this, StoreWatcher);
    this.cache = new Map();
    this.events = Object.create(Backbone.Events);

    this.triggerChange = function () {
      _this.events.trigger("change");
    };

    this.wrappedStore = wrap(this.cache, this.triggerChange, store);
  }

  StoreWatcher.prototype.on = function on(eventName, listener) {
    this.events.on(eventName, listener);
  };

  StoreWatcher.prototype.off = function off(eventName, listener) {
    this.events.off(eventName, listener);
  };

  StoreWatcher.prototype.destory = function destory() {
    this.cache.forEach(function (_ref2) {
      var destory = _ref2.destory;
      return destory();
    });
  };

  return StoreWatcher;
}();

var defaultMergeProps = function defaultMergeProps(modelProps, defaultsProps, props) {
  return _extends({}, props, modelProps, defaultsProps);
};
var hasPropsParam = function hasPropsParam(fn) {
  return fn && fn.length > 1;
};

var createGetMergedProps = function createGetMergedProps(mergeProps, propsGetter) {
  var lastArgs = [];
  var lastResult = void 0;
  return function () {
    var args = propsGetter();
    if (lastArgs.length !== args.length || args.some(function (arg, i) {
      return !shallowEqual(arg, lastArgs[i]);
    })) {
      var nextResult = mergeProps.apply(undefined, args);
      if (!shallowEqual(nextResult, lastResult)) {
        lastResult = nextResult;
      }
    }
    lastArgs = args;
    return lastResult;
  };
};

var createMapToProps = (function (store, props, mapStoreToProps, mapTriggerToProps) {
  var mergeProps = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : defaultMergeProps;

  var trigger = function trigger() {
    return store.trigger.apply(store, arguments);
  };
  var storeProps = mapStoreToProps ? mapStoreToProps(store, props) : {};
  var triggerProps = mapTriggerToProps ? mapTriggerToProps(trigger, props) : {};
  var lastProps = props;

  return {
    storeUpdated: function storeUpdated() {
      if (mapStoreToProps) {
        storeProps = mapStoreToProps(store, lastProps);
      }
    },
    updateProps: function updateProps(nextProps) {
      lastProps = nextProps;
      if (hasPropsParam(mapStoreToProps)) {
        storeProps = mapStoreToProps(store, lastProps);
      }
      if (hasPropsParam(mapTriggerToProps)) {
        triggerProps = mapTriggerToProps(trigger, lastProps);
      }
    },

    getMergedProps: createGetMergedProps(mergeProps, function () {
      return [storeProps, triggerProps, lastProps];
    })
  };
});

var index = (function (mapModelToProps, mapTriggerToProps, mergeProps) {
  return function (WrappedComponent) {
    var _Connectted$contextTy;

    var Connectted = function (_Component) {
      inherits(Connectted, _Component);

      function Connectted(props, context) {
        classCallCheck(this, Connectted);

        var _this = possibleConstructorReturn(this, _Component.call(this, props, context));

        _this.storeWatcher = new StoreWatcher(context[CONTEXT_KEY]);
        _this.mapToProps = createMapToProps(_this.storeWatcher.wrappedStore, props, mapModelToProps, mapTriggerToProps, mergeProps);

        _this.state = {
          mergedProps: _this.mapToProps.getMergedProps()
        };

        _this.storeWatcher.on("change", function () {
          _this.mapToProps.storeUpdated();
          _this.setState({
            mergedProps: _this.mapToProps.getMergedProps()
          });
        });
        return _this;
      }

      Connectted.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        if (shallowEqual(this.props, nextProps)) {
          return;
        }
        this.mapToProps.updateProps(nextProps);
        this.mapToProps.getMergedProps();
        this.setState({
          mergedProps: this.mapToProps.getMergedProps()
        });
      };

      Connectted.prototype.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
        return this.state.mergedProps !== nextState.mergedProps;
      };

      Connectted.prototype.componentWillUnmount = function componentWillUnmount() {
        this.storeWatcher.destory();
      };

      Connectted.prototype.render = function render() {
        return React__default.createElement(WrappedComponent, this.state.mergedProps);
      };

      return Connectted;
    }(React.Component);

    Connectted.displayName = "BackboneConnect(" + getDisplayName(WrappedComponent) + ")";
    Connectted.contextTypes = (_Connectted$contextTy = {}, _Connectted$contextTy[CONTEXT_KEY] = PropTypes.object, _Connectted$contextTy);
    Connectted.WrappedComponent = WrappedComponent;

    return Connectted;
  };
});

var createSelector = function createSelector() {
  for (var _len = arguments.length, selectors = Array(_len), _key = 0; _key < _len; _key++) {
    selectors[_key] = arguments[_key];
  }

  if (!selectors || selectors.length < 2) {
    throw new Error("createSelector must have at least one selector and one resultFunc");
  }
  var resultFunc = selectors.pop();
  return function (store) {
    for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    var values = selectors.map(function (selector) {
      return selector.apply(undefined, [store].concat(args));
    });
    return resultFunc.apply(undefined, values.concat(args));
  };
};

var createStructuredSelector = function createStructuredSelector(selectorObject) {
  return function () {
    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    var result = {};
    Object.keys(selectorObject).forEach(function (key) {
      result[key] = selectorObject[key].apply(selectorObject, args);
    });
    return result;
  };
};

exports.Provider = Provider;
exports.connect = index;
exports.createSelector = createSelector;
exports.createStructuredSelector = createStructuredSelector;

Object.defineProperty(exports, '__esModule', { value: true });

})));
