import Backbone from "backbone";
import StoreWatcher from "./StoreWatcher";

describe("get()", function () {
  test("Should able to get data from model", function () {
    var model = new Backbone.Model();
    var watcher = new StoreWatcher(model);
    model.set("a", 1);
    expect(watcher.wrappedStore.get("a")).toBe(1);
    model.set("a", 2);
    model.set("b", 1);
    expect(watcher.wrappedStore.get("a")).toBe(2);
    expect(watcher.wrappedStore.get("b")).toBe(1);
  });

  test("Should get the same wrapped model", function () {
    var model = new Backbone.Model();
    var nestedModel = new Backbone.Model();
    model.set("a", nestedModel);
    var watcher = new StoreWatcher(model);
    var wrappedModel1 = watcher.wrappedStore.get("a");
    var wrappedModel2 = watcher.wrappedStore.get("a");
    expect(wrappedModel1).toBe(wrappedModel2);
    wrappedModel1.set("str", "str");
    expect(wrappedModel1.get("str")).toBe(nestedModel.get("str"));
  });

  test("Should get the same wrapped collection", function () {
    var model = new Backbone.Model();
    var collection = new Backbone.Collection();
    model.set("c", collection);
    var watcher = new StoreWatcher(model);
    var wrappedCollection1 = watcher.wrappedStore.get("c");
    var wrappedCollection2 = watcher.wrappedStore.get("c");
    expect(wrappedCollection1).toBe(wrappedCollection2);
    collection.add(model);
    expect(wrappedCollection1.at(0)).toBe(watcher.wrappedStore);
  });
});

describe("on('change')", function () {
  test("Should trigger change event when modal change", function () {
    var model = new Backbone.Model();
    var watcher = new StoreWatcher(model);
    var mockCallback = jest.fn();
    watcher.on("change", mockCallback);
    model.set("a", 1);
    watcher.wrappedStore.get("a");
    expect(mockCallback.mock.calls.length).toBe(0);
    model.set("a", 2);
    expect(mockCallback.mock.calls.length).toBe(1);
  });

  test("Should trigger change event when collection change", function () {
    var model = new Backbone.Model();
    var collection = new Backbone.Collection();
    model.set("c", collection);
    var watcher = new StoreWatcher(model);
    var wrappedCollection = watcher.wrappedStore.get("c");
    var mockCallback = jest.fn();
    watcher.on("change", mockCallback);
    wrappedCollection.at(0);
    expect(mockCallback.mock.calls.length).toBe(0);
    collection.add(1);
    expect(mockCallback.mock.calls.length).toBe(1);
  });
});