var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

import shallowEqual from "../utils/shallowEqual";

var defaultMergeProps = function defaultMergeProps(modelProps, defaultsProps, props) {
  return _extends({}, props, modelProps, defaultsProps);
};
var hasPropsParam = function hasPropsParam(fn) {
  return fn && fn.length > 1;
};

var createGetMergedProps = function createGetMergedProps(mergeProps, propsGetter) {
  var lastArgs = [];
  var lastResult = void 0;
  return function () {
    var args = propsGetter();
    if (lastArgs.length !== args.length || args.some(function (arg, i) {
      return !shallowEqual(arg, lastArgs[i]);
    })) {
      var nextResult = mergeProps.apply(undefined, args);
      if (!shallowEqual(nextResult, lastResult)) {
        lastResult = nextResult;
      }
    }
    lastArgs = args;
    return lastResult;
  };
};

export default (function (store, props, mapStoreToProps, mapTriggerToProps) {
  var mergeProps = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : defaultMergeProps;

  var trigger = function trigger() {
    return store.trigger.apply(store, arguments);
  };
  var storeProps = mapStoreToProps ? mapStoreToProps(store, props) : {};
  var triggerProps = mapTriggerToProps ? mapTriggerToProps(trigger, props) : {};
  var lastProps = props;

  return {
    storeUpdated: function storeUpdated() {
      if (mapStoreToProps) {
        storeProps = mapStoreToProps(store, lastProps);
      }
    },
    updateProps: function updateProps(nextProps) {
      lastProps = nextProps;
      if (hasPropsParam(mapStoreToProps)) {
        storeProps = mapStoreToProps(store, lastProps);
      }
      if (hasPropsParam(mapTriggerToProps)) {
        triggerProps = mapTriggerToProps(trigger, lastProps);
      }
    },

    getMergedProps: createGetMergedProps(mergeProps, function () {
      return [storeProps, triggerProps, lastProps];
    })
  };
});