import Backbone from 'backbone';
import createMapToProps from "./createMapToProps";

describe('Should pass store and props to mapStoreToProps', function () {
  test('test (store, props) => ({}) function', function () {
    var store = new Backbone.Model();
    var props = {};
    var mock = jest.fn(function (store, props) {
      return {};
    });
    var mapToProps = createMapToProps(store, props, mock);
    expect(mock).toBeCalledWith(store, props);
    mapToProps.storeUpdated();
    expect(mock.mock.calls.length).toBe(2);
    var nextProps = { a: 1 };
    mapToProps.updateProps(nextProps);
    expect(mock.mock.calls.length).toBe(3);
    expect(mock).toBeCalledWith(store, nextProps);
  });

  test('test (store) => ({}) function', function () {
    var store = new Backbone.Model();
    var props = {};
    var mock = jest.fn(function (store) {
      return {};
    });
    var mapToProps = createMapToProps(store, props, mock);
    var nextProps = { a: 1 };
    mapToProps.updateProps(nextProps);
    expect(mock.mock.calls.length).toBe(1);
  });
});

describe('Should pass trigger and props to mapTriggerToProps', function () {
  test('test (trigger, props) => ({}) function', function () {
    var store = new Backbone.Model();
    var props = {};
    var mock = jest.fn(function (trigger, props) {
      return {};
    });
    var mapToProps = createMapToProps(store, props, null, mock);
    expect(mock.mock.calls[0][1]).toBe(props);
    mapToProps.storeUpdated();
    expect(mock.mock.calls.length).toBe(1);
    var nextProps = { a: 1 };
    mapToProps.updateProps(nextProps);
    expect(mock.mock.calls.length).toBe(2);
    expect(mock.mock.calls[1][1]).toBe(nextProps);
  });

  test('test (trigger) => ({}) function', function () {
    var store = new Backbone.Model();
    var props = {};
    var mock = jest.fn(function (trigger) {
      return {};
    });
    var mapToProps = createMapToProps(store, props, null, mock);
    var nextProps = { a: 1 };
    mapToProps.updateProps(nextProps);
    expect(mock.mock.calls.length).toBe(1);
  });
});

describe('Should always return same Object when no update', function () {
  test('test null function', function () {
    var mapToProps = createMapToProps(new Backbone.Model(), {});
    var result1 = mapToProps.getMergedProps();
    expect(result1).toEqual({});
    mapToProps.storeUpdated();
    mapToProps.updateProps({});
    expect(result1).toBe(mapToProps.getMergedProps());
  });

  test('test () => {} function', function () {
    var mapToProps = createMapToProps(new Backbone.Model(), {}, function () {}, function () {});
    var result1 = mapToProps.getMergedProps();
    expect(result1).toEqual({});
    mapToProps.storeUpdated();
    mapToProps.updateProps({});
    expect(result1).toBe(mapToProps.getMergedProps());
  });
});