"use strict";

exports.__esModule = true;
exports.default = undefined;

var _backbone = require("backbone");

var _backbone2 = _interopRequireDefault(_backbone);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var wrapModel = function wrapModel(curriedWrap, triggerChange, model) {
  var wrapped = Object.create(model);
  var keys = new Set();
  wrapped.get = function (propKey) {
    keys.add(propKey);
    return curriedWrap(triggerChange, model.get(propKey));
  };
  var changeListener = function changeListener(_ref) {
    var changed = _ref.changed;

    if (Object.keys(changed).some(function (key) {
      return keys.has(key);
    })) {
      triggerChange();
    }
  };

  model.on("change", changeListener);

  return {
    wrapped: wrapped,
    destory: function destory() {
      model.off("change", changeListener);
    }
  };
};

var wrapCollection = function wrapCollection(curriedWrap, triggerChange, collection) {
  var wrapped = Object.create(collection);
  wrapped.get = function (propKey) {
    return curriedWrap(triggerChange, collection.get(propKey));
  };
  wrapped.at = function (index) {
    return curriedWrap(triggerChange, collection.at(index));
  };
  wrapped.pop = function () {
    return curriedWrap(triggerChange, collection.pop());
  };
  wrapped.shift = function () {
    return curriedWrap(triggerChange, collection.shift());
  };
  wrapped.unshift = function () {
    return curriedWrap(triggerChange, collection.unshift());
  };

  collection.on("add remove reset", triggerChange);

  return {
    wrapped: wrapped,
    destory: function destory() {
      collection.off("add remove reset", triggerChange);
    }
  };
};

var wrap = function wrap(cache, triggerChange, object) {
  var wrapWithCache = function wrapWithCache(triggerChange, obj) {
    return wrap(cache, triggerChange, obj);
  };
  if (object instanceof _backbone2.default.Model) {
    return cacheable(cache, object, function () {
      return wrapModel(wrapWithCache, triggerChange, object);
    }).wrapped;
  }
  if (object instanceof _backbone2.default.Collection) {
    return cacheable(cache, object, function () {
      return wrapCollection(wrapWithCache, triggerChange, object);
    }).wrapped;
  }
  return object;
};

var cacheable = function cacheable(cache, key, fn) {
  var cached = cache.get(key);
  if (cached) {
    return cached;
  }
  var result = fn();
  cache.set(key, result);
  return result;
};

var StoreWatcher = function () {
  function StoreWatcher(store) {
    var _this = this;

    _classCallCheck(this, StoreWatcher);

    this.cache = new Map();
    this.events = Object.create(_backbone2.default.Events);

    this.triggerChange = function () {
      _this.events.trigger("change");
    };

    this.wrappedStore = wrap(this.cache, this.triggerChange, store);
  }

  StoreWatcher.prototype.on = function on(eventName, listener) {
    this.events.on(eventName, listener);
  };

  StoreWatcher.prototype.off = function off(eventName, listener) {
    this.events.off(eventName, listener);
  };

  StoreWatcher.prototype.destory = function destory() {
    this.cache.forEach(function (_ref2) {
      var destory = _ref2.destory;
      return destory();
    });
  };

  return StoreWatcher;
}();

exports.default = StoreWatcher;